package ru.tehkode.config;

import java.io.File;
import java.io.Reader;
import java.io.Writer;
import java.net.URL;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.HierarchicalConfiguration;
import org.json.simple.JSONValue;
import org.json.simple.parser.ContainerFactory;
import org.json.simple.parser.JSONParser;

public class JSONConfiguration extends SimpleConfiguration {

	private final JSONParser parser = new JSONParser();
	private boolean prettyPrint = false;
	private final ContainerFactory containerFactory = new JSONContainerFactory();

	public JSONConfiguration() {
		super();
	}

	public JSONConfiguration(HierarchicalConfiguration c) {
		super(c);
	}

	public JSONConfiguration(String fileName) throws ConfigurationException {
		super(fileName);
	}

	public JSONConfiguration(File file) throws ConfigurationException {
		super(file);
	}

	public JSONConfiguration(URL url) throws ConfigurationException {
		super(url);
	}

	@Override
	public void load(Reader in) throws ConfigurationException {
		try {
			this.loadHierarchy(this.getRootNode(), parser.parse(in, this.containerFactory));
			parser.reset(); // reset parser for futher usage
		} catch (Throwable e) {
			throw new ConfigurationException("Failed to load configuration: " + e.getMessage(), e);
		}
	}

	@Override
	public void save(Writer out) throws ConfigurationException {
		try {
			Writer jsonWriter = out;

			if (this.isPrettyPrint()) {
				jsonWriter = new JSONPrettyWriter(out);
			}

			JSONValue.writeJSONString(this.saveHierarchy(this.getRootNode()), jsonWriter);

			jsonWriter.flush();
		} catch (Throwable e) {
			throw new ConfigurationException("Failed to save configuration: " + e.getMessage(), e);
		}
	}

	/**
	 * Get status of pretty print output mode
	 *
	 * @return the value of prettyPrint
	 */
	public boolean isPrettyPrint() {
		return prettyPrint;
	}

	/**
	 * Set pretty print output mode
	 *
	 * @param prettyPrint
	 */
	public void setPrettyPrint(boolean prettyPrint) {
		this.prettyPrint = prettyPrint;
	}

	protected class JSONContainerFactory implements ContainerFactory {

		@Override
		public List creatArrayContainer() {
			return new LinkedList();
		}

		@Override
		public Map createObjectContainer() {
			return new LinkedHashMap();
		}	
	}
}
